@@ -45,6 +45,13 @@ static GObjectClass *parent_class = NULL;
  * GdaDataModelHash class implementation
  */
 
+static gint
+gda_data_model_hash_get_n_rows (GdaDataModel *model)
+{
+	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), -1);
+	return g_hash_table_size (GDA_DATA_MODEL_HASH (model)->priv->rows);
+}
+
 static gint
 gda_data_model_hash_get_n_columns (GdaDataModel *model)
 {
@@ -61,7 +68,7 @@ gda_data_model_hash_describe_column (GdaDataModel *model, gint col)
 }
 
 static gboolean
-gda_data_model_hash_is_editable (GdaDataModel *model)
+gda_data_model_hash_is_updatable (GdaDataModel *model)
 {
 	g_return_val_if_fail (GDA_IS_DATA_MODEL_HASH (model), FALSE);
 	return TRUE;
@@ -116,11 +123,12 @@ gda_data_model_hash_class_init (GdaDataModelHashClass *klass)
 	parent_class = g_type_class_peek_parent (klass);
 
 	object_class->finalize = gda_data_model_hash_finalize;
+	model_class->get_n_rows = gda_data_model_hash_get_n_rows;
 	model_class->get_n_columns = gda_data_model_hash_get_n_columns;
 	model_class->describe_column = gda_data_model_hash_describe_column;
 	model_class->get_row = gda_data_model_hash_get_row;
 	model_class->get_value_at = gda_data_model_hash_get_value_at;
-	model_class->is_editable = gda_data_model_hash_is_editable;
+	model_class->is_updatable = gda_data_model_hash_is_updatable;
 	model_class->append_row = gda_data_model_hash_append_row;
 	model_class->remove_row = gda_data_model_hash_remove_row;
 }
